#!/bin/bash

DTBO_PATH="${DTBO_PATH:-/run/media/mmcblk1p1/dtbos}"


# Help text function
show_help() {
    echo "Usage: $0 -m <camera_model> -o <operation>"
    echo "Options:"
    echo "  -m, --model      Specify the camera model, currently supported: rpi13, rpi21, vk000435"
    echo "  -o, --operation  Specify the operation (video or snapshot)"
    echo "  -h, --help       Show this help message"
    exit 1
}

# Parse command line options
while getopts "m:o:h" opt; do
    case $opt in
        m|--model)
            camera_model="$OPTARG"
            ;;
        o|--operation)
            operation="$OPTARG"
            ;;
        h|--help)
            show_help
            ;;
        *)
            echo "Invalid option: -$opt" >&2
            show_help
            ;;
    esac
done

# Check if required parameters are provided
if [ -z "$camera_model" ] || [ -z "$operation" ]; then
    echo "Error: Both camera model and operation must be specified."
    show_help
    exit 1
fi

i2c_bus="4"

case $camera_model in
    
    rpi21)
      #RasPI Cam 2.1
      sensor="imx219"
      i2c_addr="10"
      format="SRGGB10"
      ;;
    rpi13)
      #RasPI Cam 1.3
      sensor="ov5647"
      i2c_addr="36"
      format="SBGGR10"
      ;;
    vk000435)
      #Vision Components VK000435
      sensor="imx290"
      i2c_addr="1a"
      format="SRGGB10"
      ;;
    vcother)
      #Vision Components OV9281
      #monochrome, does not work so far
      sensor="ov9282" #sic!
      i2c_addr="60"
      format="Y10"
      ;;
    *)
        echo "Error: Invalid model."
        show_help
        exit 1
        ;;
esac

addr="${i2c_bus}-00${i2c_addr}"


OV_PATH="/sys/kernel/config/device-tree/overlays/cam"

if [ ! -d "${OV_PATH}" ]; then
  #check for i2c-addr before loading overlay, otherwise kernel might crash
  result=$(i2cdetect -r -y $i2c_bus | grep -w " $i2c_addr")

  if [[ -z "$result" ]]; then
      echo "Error: cam not present on i2c bus, not continuing"
      exit 1
  fi

  

  if [ -f ${DTBO_PATH}/${sensor}-overlay.dtbo ]; then
    mkdir $OV_PATH
    cat ${DTBO_PATH}/${sensor}-overlay.dtbo  > ${OV_PATH}/dtbo
  else
    echo "${DTBO_PATH}/${sensor}-overlay.dtbo was not found"
    exit 1
  fi
fi

BIND_ADDR=0.0.0.0
BIND_PORT=5001
DEV=/dev/media0

x_in=1920
y_in=1080
x_out=1920
y_out=1080

sensor_addr="${sensor} ${addr}"

format_in=${format}_1X10/${x_in}x${y_in}
format_demosaic_out=RBG24/${x_in}x${y_in}
format_out=UYVY/${x_out}x${y_out}

echo "Setting up cam"
media-ctl --set-v4l2 "'${sensor_addr}'                 :0[fmt:$format_in field:none]"  -d ${DEV}
media-ctl --set-v4l2 "'a4050000.mipi_csi2_rx_subsystem':1[fmt:$format_in field:none]"  -d ${DEV}

echo "Setting up rx"
media-ctl --set-v4l2 "'a4050000.mipi_csi2_rx_subsystem':0[fmt:$format_in field:none]"  -d ${DEV}
media-ctl --set-v4l2 "'a4060000.v_demosaic'            :0[fmt:$format_in field:none]"  -d ${DEV}

echo "Setting up demosaic"
media-ctl --set-v4l2 "'a4060000.v_demosaic' :1[fmt:$format_demosaic_out field:none]"   -d ${DEV}
media-ctl --set-v4l2 "'a4080000.v_gamma_lut':0[fmt:$format_demosaic_out field:none]"   -d ${DEV}

echo "Setting up gamma"
media-ctl --set-v4l2 "'a4080000.v_gamma_lut':1[fmt:$format_demosaic_out field:none]"   -d ${DEV}
media-ctl --set-v4l2 "'a4040000.v_proc_ss'  :0[fmt:$format_demosaic_out field:none]"   -d ${DEV}

echo "Setting up ss1"
media-ctl --set-v4l2 "'a4040000.v_proc_ss':1[fmt:$format_out field:none]"              -d ${DEV}
media-ctl --set-v4l2 "'a4000000.v_proc_ss':0[fmt:$format_out field:none]"              -d ${DEV}
  
echo "Setting up ss2"  
media-ctl --set-v4l2 "'a4000000.v_proc_ss':1[fmt:$format_out field:none]"              -d ${DEV}



case $operation in
    video)
        echo "Starting video capture for camera model: $camera_model"
        gst-launch-1.0 v4l2src device=/dev/video0 ! "video/x-raw, width=${x_out}, height=${y_out}, format=UYVY, framerate=10/1" ! jpegenc ! multipartmux ! tcpserversink host=$BIND_ADDR port=$BIND_PORT sync=false

        ;;
    snapshot)
        echo "Taking snapshot for camera model: $camera_model"
        yavta -s ${x_out}x${y_out} -n1 -c1 -F /dev/video0
        if [ -f frame-000000.bin ]; then
          exit 0
        else
          exit 1
        fi
        ;;
    *)
        echo "Error: Invalid operation. Choose either 'video' or 'snapshot'."
        show_help
        ;;
esac